//
// EIOWriteTest.cpp
//
//
// This simple application configures the EIO driver as outputs
// and runs a walking-ones test down the bits.  It displays the 
// output values on the debug port.
//
// By comparing input readings to the output values set, the 
// application reports if bits are stuck low ("v") or // high ("^").
//
// Applied Data Systems
//
// 29Aug2003 720020-11561 ak
// - initial release
//

#include "stdafx.h"
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include "eio_ctrl.h"


//-------------------------------------------------------------------
//-------------------------------------------------------------------
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	DWORD dwReading, dwNumBytes, dwBytesRead, dwOutputValue;
	DWORD dwBitIn, dwBitOut;
	DWORD mask = 0x03FF;   // EIO1: all outputs
	HANDLE hEIO;
	BOOL Slept;
	int i, j=0;

    RETAILMSG(TRUE,(L"\r\nEIOWriteTest 720020-11561:\r\n"));
    RETAILMSG(TRUE,(L"Runs walking-ones test on EIO outputs. Displays readings on debug port.\r\n"));

	// Open and initialize port
	hEIO = CreateFile(TEXT("EIO1:"), GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ,NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if(hEIO==INVALID_HANDLE_VALUE)
	{
		RETAILMSG(TRUE,(_T("Error: Unable to open EIO1: (error # %d)\r\n"),GetLastError()));
		RETAILMSG(TRUE,(_T("Program aborted.\r\n")));
		return 1;
	}
	
	if(!DeviceIoControl(hEIO,IOCTL_EIO_CONFIG_PORTS,&mask,sizeof(DWORD),NULL,0,&dwNumBytes,NULL ))
	{
		RETAILMSG(TRUE,(_T("Error: Unable to set direction of EIO1: (error # %d)\r\n"),GetLastError()));
		RETAILMSG(TRUE,(_T("Program aborted.\r\n")));
		return 2;
	}
    
	// Prepare to display data
	RETAILMSG(TRUE,(L"Bit #:  9 8 7 6 5 4 3 2 1 0\r\n"));
	dwOutputValue = 0x001;

	// Run indefinitely
	while(TRUE)
	{		
		// return cursor to start of line
		// display spinning character to indicate activity
		j++;
		switch (j%4)  // returns 0-3
		{
			case 0 : RETAILMSG(TRUE, (_T("\r    -  "))); break;
			case 1 : RETAILMSG(TRUE, (_T("\r    \\  "))); break;
			case 2 : RETAILMSG(TRUE, (_T("\r    |  "))); break;
			case 3 : RETAILMSG(TRUE, (_T("\r    /  "))); break;
		}

		// update ouptput value
		if (dwOutputValue == 0x001)
			dwOutputValue = 0x200;
		else
			dwOutputValue = dwOutputValue >> 1; // shift active bit down

		// set ouptput value
		WriteFile(hEIO, &dwOutputValue, sizeof(dwReading), &dwBytesRead,NULL);

		// read the data
		ReadFile(hEIO, &dwReading, sizeof(dwReading), &dwBytesRead,NULL);

		// display the data in binary format
		for(i=9; i>=0; i--)
		{
			dwBitOut = (dwOutputValue >>i) & 0x01;
			dwBitIn  = (dwReading     >>i) & 0x01;

			if (dwBitIn == dwBitOut)
				RETAILMSG(TRUE, (_T("%2d"), dwBitIn));
			else if (dwBitIn > dwBitOut)
			{
				RETAILMSG(TRUE, (_T(" ^")));  // bit is stuck high
				Sleep(250);
				Slept=TRUE;
			}
			else
			{
				RETAILMSG(TRUE, (_T(" v")));  // bit is stuck low
				Sleep(250);
				Slept=TRUE;
			}
		}

		// display in hex, too
		RETAILMSG(TRUE, (_T("  (out=0x%03X)  "), dwOutputValue));

		if (!Slept)
			Sleep(100);
		Slept=FALSE;
	}

	RETAILMSG(TRUE,(L"\r\nEIOWriteTest: Complete (should never get here).\r\n"));

	return 0;

}

